/* Copyright (c) 2008, Florent FAYOLLE


This file is part of Zenith

Zenith is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/* Implementation file */
#include "Him.h"
#define	MIN(x,y)	(((x)<(y))?(x):(y))
#define	MAX(x,y)	(((x)>(y))?(x):(y))
#define HIM_NORMAL	0
#define HIM_CANCEL_ACTION	1
#define HIM_CONTINUE	2
#define MAX_MOVE	5 // maximum of pixel that a mouse can be moved


//using namespace nsIProxyObjectManager;
NS_IMPL_ISUPPORTS1(Him, IHim)
Him::Him()
{
	
	regX = regY = -1;
	display = XOpenDisplay(NULL);
	
  /* member initializers and constructor code */
}

Him::~Him()
{
	XCloseDisplay(display);
  /* destructor code */
}



/* void MoveCursorTo (in unsigned short posX, in unsigned short posY); */
NS_IMETHODIMP Him::MouseMove(PRInt16 posX, PRInt16 posY, PRInt16 *status)
{
	if (display == NULL){
	    perror("display == NULL (Function : Him.MoveCursorTo)");
	}
	int min = MIN(abs(posX),abs(posY)), i, a=((posX>0)?1:-1), b=((posY>0)?1:-1), steps = 0;

	// we see where the pointer is in the screen
	XQueryPointer(display, RootWindow(display, DefaultScreen(display)), &event.xbutton.root, &event.xbutton.window, &event.xbutton.x_root, &event.xbutton.y_root, &event.xbutton.x, &event.xbutton.y, &event.xbutton.state);
	for(i=0; i<min;i++){
		
		SetCursorTo(a,b);
		
		if(detectMouseMove()){
			regX = -1;
			regY = -1;
			*status = HIM_CANCEL_ACTION;
			return NS_OK;
		}
		Msleep(4);
			
	}
	a*=(abs(posX)>abs(posY)); b*=(abs(posX)<abs(posY));//abs(a)=1 && b=0 if x>y, abs(b)=1 && a=0 if y>x
	for(i=0; i<MAX(abs(posX),abs(posY))-min;i++){
		SetCursorTo(a,b);
		
		if(detectMouseMove()){
			regX = -1;
			regY = -1;
			*status = HIM_CANCEL_ACTION;
			return NS_OK;
		}
		Msleep(4);
	}
	regX = regY = -1;
	*status = HIM_NORMAL;
	XSendEvent(display, PointerWindow, True, 0xfff, &event);
	return NS_OK;
}

NS_IMETHODIMP Him::SetCursorTo(PRInt16 x, PRInt16 y){
	//Display *displayMain = XOpenDisplay(NULL);
	//XFlush(display);
	if(display == NULL)
	{
		perror("Erreur SetCursorTo\n");
		return NS_ERROR_FAILURE;
	}

	XWarpPointer(display, RootWindow(display, DefaultScreen(display)), None, 0, 0, 0, 0, (int)x, (int)y);
	return NS_OK;

	//XCloseDisplay(display);
}

NS_IMETHODIMP Him::DetectMouseMove(int * res){
	*res = detectMouseMove();
	return NS_OK;
}

bool Him::detectMouseMove(){
	int curX, curY;
	XQueryPointer(display, RootWindow(display, DefaultScreen(display)), &event.xbutton.root, &event.xbutton.window, &curX, &curY, &event.xbutton.x, &event.xbutton.y, &event.xbutton.state);
	if(regX>0 && regY>0 && (abs(curX-regX) >MAX_MOVE || abs(curY-regY)>MAX_MOVE)){
		regY = regX = -1;
		return true;
	}
	regX = curX;
	regY = curY;
	return false;
}
NS_IMETHODIMP Him::ResetReg(){
	regX = regY = -1;
	return NS_OK;
}
NS_IMETHODIMP Him::Msleep(PRInt16 time){
  usleep(time*1000);
  return NS_OK;
}

NS_IMETHODIMP Him::Msleep2(PRInt16 time, PRInt16* status){
	int i;
	for(i=0; i<time/100; i++){
		Msleep(100);
		if(detectMouseMove()){
			*status =  HIM_CANCEL_ACTION; 
			return NS_OK;
		}
	}
	Msleep(time- 100*((int)time/100));
	return NS_OK;
}

NS_IMETHODIMP Him::GetMousePosition(PRInt16* posX, PRInt16* posY){
	XQueryPointer(display, RootWindow(display, DefaultScreen(display)), &event.xbutton.root, &event.xbutton.window, &event.xbutton.x_root, &event.xbutton.y_root, &event.xbutton.x, &event.xbutton.y, &event.xbutton.state);
	*posX = event.xbutton.x_root;
	*posY = event.xbutton.y_root;
	return NS_OK;
}

NS_IMETHODIMP Him::Click(PRInt16 button){                                                                      
        //Display *display = XOpenDisplay(NULL);     
        fflush(stdout);
	//XEvent event;
        if(display == NULL)
        {                  
                fprintf(stderr, "Erreur mouseClick !!!\n");
                return NS_ERROR_FAILURE;                  
        }                                                  

        memset(&event, 0x00, sizeof(event));

        event.type = ButtonPress;
        event.xbutton.button = button;
        event.xbutton.same_screen = True;

        XQueryPointer(display, RootWindow(display, DefaultScreen(display)), &event.xbutton.root, &event.xbutton.window, &event.xbutton.x_root, &event.xbutton.y_root, &event.xbutton.x, &event.xbutton.y, &event.xbutton.state);

        event.xbutton.subwindow = event.xbutton.window;

        while(event.xbutton.subwindow)
        {
                event.xbutton.window = event.xbutton.subwindow;
                XQueryPointer(display, event.xbutton.window, &event.xbutton.root, &event.xbutton.subwindow, &event.xbutton.x_root, &event.xbutton.y_root, &event.xbutton.x, &event.xbutton.y, &event.xbutton.state);
        }
	XSendEvent(display, PointerWindow, True, 0xfff, &event);
        XFlush(display);

        usleep(10000);

        event.type = ButtonRelease;
        event.xbutton.state = 0x100;
	XSendEvent(display, PointerWindow, True, 0xfff, &event);
        XFlush(display);

        //XCloseDisplay(display);

	return NS_OK;
}